/******************************************************************************/
/* Sistemas Integrados TECNISA                                                */
/*                                                                            */
/* Compras e Suprimentos                                                      */
/*                                                                            */
/******************************************************************************/
/*                                                                            */
/* Nome: 8180_SUPSTP_CONSOLIDA_SOLICITA                                       */
/*                                                                            */
/* Descrio:                                                                 */
/* Consolida em uma solicitao outras solicitaes que estejam nos           */
/* parmetros informados.                                                     */
/*                                                                            */
/*                                                                            */
/* Criao:                                                                   */
/* 21/08/2000 - Adriano de Arajo                                             */
/*                                                                            */
/* Manutenes:                                                               */
/*   /  /   -                                                                 */
/*                                                                            */
/*                                                                            */
/******************************************************************************/

/******************************************************************************/
/* PARAMETROS:                                                                */
/* @P_CDFILCONS     - Codigo da Filial para Consolidao                      */
/* @P_CDFILIAL      - Codigo da Filial.                                       */
/* @P_CDFILEST      - Codigo da Filial para considerar o estoque.             */
/* @P_CDCOMPRADOR   - Cdigo do Comprador                                     */
/* @P_DTENTRINI     - Data de Entrega Inicial                                 */
/* @P_DTENTRFIN     - Data de Entrega Final                                   */
/* @P_CDPRODINI     - Produto Inicial                                         */
/* @P_CDPRODFIN     - Produto Final                                           */
/* @P_IDCONSESTOQ   - Considera Estoque                                       */
/* @P_IDCONSOLIDA   - Consolida todas ou somente as nao consolidadas          */
/* @P_IDPRODORIG    - Considera Produto Origem(Matria Prima)                 */
/* @P_NRSOLICMP     - Numero da Solicitao de retorno                        */
/******************************************************************************/

IF EXISTS (SELECT * FROM SYSOBJECTS O, SYSUSERS U
                    WHERE
	        O.NAME = 'CONSOLIDA_SOLICITA' AND
	        TYPE = 'P'                    AND
	        U.UID = O.UID                 AND
	        U.NAME = USER)

   DROP PROCEDURE CONSOLIDA_SOLICITA
   GO

CREATE PROCEDURE CONSOLIDA_SOLICITA
   @P_CDFILCONS     VARCHAR(4),
   @P_CDFILCENT     VARCHAR(4),
   @P_CDFILIAL      VARCHAR(2000) = NULL,
   @P_CDFILEST      VARCHAR(4),
   @P_CDCOMPRADOR   VARCHAR(2000) = NULL,
   @P_DTENTRINI     DATETIME,
   @P_DTENTRFIN     DATETIME,
   @P_CDPRODINI     VARCHAR(20),
   @P_CDPRODFIN     VARCHAR(20),
   @P_IDCONSESTOQ   VARCHAR(1),
   @P_IDCONSOLIDA   VARCHAR(1),
   @P_IDPRODORIG    VARCHAR(1),
   @P_DTPROCESS     DATETIME,
   @P_LISTA         VARCHAR(10),
   @P_CONSSOL       VARCHAR(1),
   @P_CONSAF        VARCHAR(1),
   @P_CONSLIMSEG    VARCHAR(1) = 'N',
   @P_CONSOL        VARCHAR(1) = 'N',
   @P_NRSOLICMP     VARCHAR(10) OUTPUT

AS
   DECLARE @V_NRPROXSOLI   VARCHAR(20);

   DECLARE @V_QTATUEST     DOUBLE PRECISION;
   DECLARE @V_QTATUESTAUX  DOUBLE PRECISION;
   DECLARE @V_IDPRIMVEZ    BIT;                  -- 1 = TRUE, 0 = FALSE
   DECLARE @V_CONTA        INT;

   DECLARE @V_TCDFILIAL    VARCHAR(1);
   DECLARE @V_TCDCOMPRADOR VARCHAR(1);

   DECLARE @V_ITEMS_CDFILSOLI    VARCHAR(4);
   DECLARE @V_ITEMS_CDFILLAN     VARCHAR(4);
   DECLARE @V_ITEMS_NRSOLICMP    VARCHAR(10);
   DECLARE @V_ITEMS_CDPRODUTO    VARCHAR(10);
   DECLARE @V_ITEMS_DTPROGENTR   DATETIME;
   DECLARE @V_ITEMS_VRFATOCONV   DOUBLE PRECISION;
   DECLARE @V_ITEMS_QTPRODESTO   NUMERIC;
   DECLARE @V_ITEMS_CDPRODESTO   VARCHAR(10);
   DECLARE @V_ITEMS_CDPRODORIG   VARCHAR(10);
   DECLARE @V_ITEMS_CDESTPROD    VARCHAR(10);
   DECLARE @V_ITEMS_CDEXISTEPROD VARCHAR(10);

   DECLARE @V_ITEM_CDFILSOLI     VARCHAR(4);
   DECLARE @V_ITEM_CDFILLAN      VARCHAR(4);
   DECLARE @V_ITEM_NRSOLICMP     VARCHAR(10);
   DECLARE @V_ITEM_CDPRODUTO     VARCHAR(10);
   DECLARE @V_ITEM_DTPROGENTR    DATETIME;
   DECLARE @V_ITEM_QTSOLIPROD    DOUBLE PRECISION;
   DECLARE @V_ITEM_VRFATOCONV    DOUBLE PRECISION;
   DECLARE @V_ITEM_VRARREDCMP    DOUBLE PRECISION;

   DECLARE @V_NOVO_CODIGO  VARCHAR(100);
   DECLARE @V_DTPROCESSA   DATETIME;
   DECLARE @V_IDUTILDTPROC VARCHAR(1);

   DECLARE @V_ANTPRODESTO VARCHAR(10);

   DECLARE @V_CNVPRODUNEST_CDPRODUTO  VARCHAR(10);
   DECLARE @V_CNVPRODUNEST_VRFATUNEST DOUBLE PRECISION;

   DECLARE @V_VRARREDCMP DOUBLE PRECISION;

   DECLARE @V_QTSOLI DOUBLE PRECISION;

   DECLARE @V_QTSOL  DOUBLE PRECISION;
   DECLARE @V_QTAF   DOUBLE PRECISION;

   DECLARE @V_PERIODNCALC_CDPERIENTR VARCHAR(2);
   DECLARE @V_PERIODNCALC_DTPREVCONS DATETIME;

   DECLARE @V_ITEMANTECIP_CDPRODUTO   VARCHAR(10);
   DECLARE @V_ITEMANTECIP_DTPROGENTR  DATETIME;
   DECLARE @V_ITEMANTECIP_QTSOLIPROD  DOUBLE PRECISION;
   DECLARE @V_ITEMANTECIP_CDARVPROD   VARCHAR(20);
   DECLARE @V_ITEMANTECIP_CDPERIENTR  VARCHAR(2);

   DECLARE @V_NRDIASEG     DOUBLE PRECISION;
   DECLARE @V_NRLIMISEGCAD VARCHAR(3);
   DECLARE @V_NOVADATA     DATETIME;
   DECLARE @V_DTENTREGA    DATETIME;

   DECLARE @V_CDPRODFIN    VARCHAR(20);
   DECLARE @V_CDFILIOPELOG VARCHAR(4);
   DECLARE @V_VRFATOCONV   DOUBLE PRECISION;
BEGIN
  SET @V_IDPRIMVEZ = 1;
  SET @P_NRSOLICMP = NULL;

  IF @P_CDFILIAL IS NULL BEGIN
    SET @V_TCDFILIAL = 'T';
  END;
  IF @P_CDCOMPRADOR IS NULL BEGIN
    SET @V_TCDCOMPRADOR = 'T';
  END;

  SET @V_ANTPRODESTO = ' '; /*A varivel deve receber espao, no mude isso */

  SET @V_CDPRODFIN = (SUBSTRING(@P_CDPRODFIN+(REPLICATE('Z', 20)), 1, 20));

  DECLARE C_ITEMS CURSOR FOR
     SELECT
      ITEMSOLI.CDFILSOLI, ITEMSOLI.CDFILLAN, ITEMSOLI.NRSOLICMP,
      ITEMSOLI.CDPRODUTO, ITEMSOLI.DTPROGENTR, PRODUTO.VRFATOCONV,
      (ITEMSOLI.QTSOLIPROD * (1 + ISNULL(PRODUTO.VRFATACRCMP, 0) + CASE @P_IDPRODORIG
                                                                      WHEN 'S' THEN ISNULL(PRODUTO.VRFATPERDCMP, 0)
                                                                      ELSE 0
                                                                   END)) * PRODUTO.VRFATOCONV AS QTPRODESTO,
      PRODUTO.CDPRODESTO,
      CASE @P_IDPRODORIG    /* ESTE CAMPO FOI CRIADO PARA QUE SE VERIFIQUE O ESTOQUE UMA VEZ PARA CADA PRODUTO DE ESTOQUE OU UMA VEZ PARA CADA PRODUTO DE ORIGEM DEPENDENDO DE @P_IDPRODORIG */
         WHEN 'S' THEN ISNULL(PRODUTO.CDPRODORIG, PRODUTO.CDPRODESTO)
         ELSE PRODUTO.CDPRODESTO
      END AS CDPRODORIG,
      PRODUTO.CDPRODORIG AS CDEXISTEPRODORIG,
      CASE @P_IDPRODORIG    /* ESTE CAMPO FOI CRIADO PARA SER USADO NA VERIFICACAO DO CADASTRO DE CONVERSAO PARA UNIDADE DE ESTOQUE */
         WHEN 'S' THEN ISNULL(PRODUTO.CDPRODORIG, PRODUTO.CDPRODUTO)
         ELSE PRODUTO.CDPRODUTO
      END AS CDESTPROD
     FROM
      ITEMSOLI,
      PRODUTO,
      SOLICITA,
      (SELECT CDLISTA FROM LISTA WHERE NRLISTA = @P_LISTA
       UNION ALL
       SELECT '1' AS CDLISTA FROM PARAMGERAL WHERE @P_LISTA IS NULL) LI
     WHERE
      ((SOLICITA.CDFILSOLI = @P_CDFILCONS AND SOLICITA.IDTIPLAN <> 'C') OR
       (SOLICITA.CDFILSOLI <> @P_CDFILCONS)) AND

      (ITEMSOLI.CDFILSOLI = SOLICITA.CDFILSOLI)  AND
      (ITEMSOLI.CDFILLAN  = SOLICITA.CDFILLANCA) AND
      (ITEMSOLI.NRSOLICMP = SOLICITA.NRSOLICMP)  AND

      (CHARINDEX(ITEMSOLI.CDFILSOLI, @P_CDFILIAL) <> 0 OR @V_TCDFILIAL = 'T') AND
      (ITEMSOLI.DTPROGENTR BETWEEN @P_DTENTRINI AND @P_DTENTRFIN) AND
      (PRODUTO.CDARVPROD BETWEEN @P_CDPRODINI AND @V_CDPRODFIN) AND

      (SOLICITA.CDFILLANCA+SOLICITA.CDFILSOLI+SOLICITA.NRSOLICMP = LI.CDLISTA OR @P_LISTA IS NULL) AND
      EXISTS
       (SELECT PC.CDOPERADOR
        FROM
          PRODUTO PR, PRODCOMP PC
        WHERE
          PC.CDPRODUTO = PR.CDPRODUTO AND
         (CHARINDEX(PC.CDOPERADOR, @P_CDCOMPRADOR) <> 0 OR @V_TCDCOMPRADOR = 'T') AND
         (PRODUTO.CDARVPROD LIKE PR.CDARVPROD +'%')) AND

      (ITEMSOLI.IDITMCONS = 'N' OR @P_IDCONSOLIDA = 'T') AND
      (ITEMSOLI.CDPRODUTO = PRODUTO.CDPRODUTO) AND
      (ISNULL(ITEMSOLI.IDSOLIAPROV,'S') = 'S') AND
      ((ISNULL(ITEMSOLI.CDFILIOPELOG,'X') = @P_CDFILCONS AND @P_CONSOL = 'S') OR (@P_CONSOL = 'N'))
     ORDER BY
     	 CASE @P_IDPRODORIG
          WHEN 'S' THEN ISNULL(PRODUTO.CDPRODORIG, PRODUTO.CDPRODESTO)
          ELSE PRODUTO.CDPRODESTO
       END,
       ITEMSOLI.DTPROGENTR,
       PRODUTO.VRFATOCONV

  OPEN C_ITEMS;
  FETCH NEXT FROM C_ITEMS INTO @V_ITEMS_CDFILSOLI,  @V_ITEMS_CDFILLAN,   @V_ITEMS_NRSOLICMP,
                               @V_ITEMS_CDPRODUTO,  @V_ITEMS_DTPROGENTR, @V_ITEMS_VRFATOCONV,
                               @V_ITEMS_QTPRODESTO, @V_ITEMS_CDPRODESTO, @V_ITEMS_CDPRODORIG,
                               @V_ITEMS_CDEXISTEPROD, @V_ITEMS_CDESTPROD;

  WHILE (@@FETCH_STATUS = 0) BEGIN
    /* ARREDONDA A QUANTIDADE DO PRODUTO */
    SELECT
       @V_VRARREDCMP = ISNULL(MAX(VRARREDCMP), 0),
       @V_VRFATOCONV = ISNULL(MAX(VRFATOCONV), 0)
    FROM
       PRODUTO
    WHERE
       CDPRODUTO = @V_ITEMS_CDPRODUTO;

    IF @V_VRARREDCMP > 0 BEGIN
       IF (((CAST(@V_ITEMS_QTPRODESTO AS NUMERIC) / (@V_VRARREDCMP*@V_VRFATOCONV)) - (ROUND(@V_ITEMS_QTPRODESTO / @V_VRARREDCMP, 0, 1))) <> 0)
       BEGIN
          SET @V_ITEMS_QTPRODESTO = CEILING(CAST(@V_ITEMS_QTPRODESTO AS NUMERIC) / @V_VRARREDCMP) * @V_VRARREDCMP;
       END;
    END;

   /* SE DEVE CONSIDERAR O ESTOQUE */
    IF @V_ITEMS_CDPRODORIG <> @V_ANTPRODESTO BEGIN
      SET @V_QTATUEST  = 0;

      SET @V_DTPROCESSA   = @P_DTPROCESS;
      SET @V_IDUTILDTPROC = NULL;
      SELECT
         @V_DTPROCESSA   = ISNULL(MAX(PARAMFILIAL.DTPROCESSA), @P_DTPROCESS),
         @V_IDUTILDTPROC = MAX(PARAMGERAL.IDUTILDTPROC)
      FROM
         PARAMGERAL,
         PARAMFILIAL
      WHERE
         PARAMFILIAL.CDFILIAL = @P_CDFILEST;
      IF @V_IDUTILDTPROC = 'N' BEGIN
         SET @V_DTPROCESSA = @P_DTPROCESS;
      END;
      IF @P_IDCONSESTOQ = 'S' BEGIN /* BUSCA O ESTOQUE DO PRODUTO AT A DATA DE PROCESSAMENTO */
         DECLARE C_CNVPRODUNEST CURSOR FOR
             SELECT
                ISNULL(CDPRUNEST, PRODUTO.CDPRODESTO) AS CDPRODUTO,
                ISNULL(VRFATUNEST, 1) AS VRFATUNEST
             FROM
             	 PRODUTO LEFT JOIN CNVPRODUNEST
                        ON PRODUTO.CDPRODUTO = CNVPRODUNEST.CDPRODUTO
             WHERE
                PRODUTO.CDPRODUTO = @V_ITEMS_CDESTPROD;

         OPEN C_CNVPRODUNEST;
         FETCH NEXT FROM C_CNVPRODUNEST INTO @V_CNVPRODUNEST_CDPRODUTO,
                                             @V_CNVPRODUNEST_VRFATUNEST;
         WHILE (@@FETCH_STATUS = 0)
         BEGIN
            EXEC CALC_ESTOQ_PREV @P_CDFILEST, @V_CNVPRODUNEST_CDPRODUTO, @V_DTPROCESSA, @P_CALC_ESTOQ_PREV = @V_QTATUESTAUX OUTPUT;
            SET @V_QTATUEST = @V_QTATUEST + (@V_QTATUESTAUX * @V_CNVPRODUNEST_VRFATUNEST);
            FETCH NEXT FROM C_CNVPRODUNEST INTO @V_CNVPRODUNEST_CDPRODUTO,
                                                @V_CNVPRODUNEST_VRFATUNEST;
         END;
         CLOSE C_CNVPRODUNEST;
         DEALLOCATE C_CNVPRODUNEST;
      END;
      IF @P_CONSSOL = 'S' BEGIN
         SELECT
            @V_QTSOL = SUM(ITEMSOLI.QTSOLIPROD*PRODUTO.VRFATOCONV/P.VRFATOCONV)
         FROM
            ITEMSOLI, PRODUTO, PRODUTO P
         WHERE
            ITEMSOLI.CDFILSOLI = @P_CDFILCONS AND
            ITEMSOLI.CDPRODUTO = PRODUTO.CDPRODUTO AND
            ITEMSOLI.DTPROGENTR BETWEEN @V_DTPROCESSA AND @P_DTENTRINI - 1 AND
            PRODUTO.CDPRODESTO = P.CDPRODESTO AND
            P.CDPRODUTO = @V_ITEMS_CDESTPROD AND
            ITEMSOLI.IDITMCONS = 'N' AND
            (ISNULL(ITEMSOLI.IDSOLIAPROV,'S') = 'S');
         SET @V_QTATUEST = @V_QTATUEST - ISNULL(@V_QTSOL, 0);
      END;
      IF @P_CONSAF = 'S' BEGIN
         SELECT
            @V_QTAF = SUM(ITEMAUTF.QTPROGENTAF*PRODUTO.VRFATOCONV/P.VRFATOCONV)
         FROM
            PRODUTO, ITEMAUTF, AUTOFORNE, PRODUTO P
         WHERE
            AUTOFORNE.CDFILDEST = ITEMAUTF.CDFILDEST AND
            AUTOFORNE.CDFILLANC = ITEMAUTF.CDFILLANC AND
            AUTOFORNE.NRAUTOFORN = ITEMAUTF.NRAUTOFORN AND
            ITEMAUTF.CDPRODUTO = PRODUTO.CDPRODUTO AND
            PRODUTO.CDPRODESTO = P.CDPRODESTO AND
            P.CDPRODUTO = @V_ITEMS_CDESTPROD AND
            ITEMAUTF.CDFILDEST = @P_CDFILCONS AND
            ITEMAUTF.DTPROGENTAF BETWEEN @V_DTPROCESSA AND @P_DTENTRINI - 1 AND
            AUTOFORNE.IDSITUATUAAF IS NULL AND
            AUTOFORNE.IDAFFORALIM = 'N' AND
            AUTOFORNE.IDAUTOFORNEX = 'N' AND
            ISNULL(AUTOFORNE.IDBAIXAAF,'0') <> '5';
         SET @V_QTATUEST = @V_QTATUEST + ISNULL(@V_QTAF, 0);
      END;

      /* Considera fator de converso do produto origem */
      IF (@P_IDPRODORIG = 'S') AND (@V_ANTPRODESTO <> ' ') BEGIN
         DECLARE C_ITEM CURSOR FOR
             SELECT
                ITEMSOLI.CDFILSOLI,
                ITEMSOLI.CDFILLAN,
                ITEMSOLI.NRSOLICMP,
                ITEMSOLI.CDPRODUTO,
                ITEMSOLI.DTPROGENTR,
                ITEMSOLI.QTSOLIPROD,
                ISNULL(PRODUTO.VRFATOCONV, 1) VRFATOCONV,
                ISNULL(VRARREDCMP, 0) VRARREDCMP
             FROM
                ITEMSOLI, PRODUTO
             WHERE ITEMSOLI.CDPRODUTO = PRODUTO.CDPRODUTO
               AND ITEMSOLI.CDFILSOLI = @P_CDFILCONS
               AND ITEMSOLI.CDFILLAN  = @P_CDFILCENT
               AND ITEMSOLI.NRSOLICMP = SUBSTRING(@V_NRPROXSOLI, 11, 10)
               AND ITEMSOLI.CDPRODUTO = @V_ANTPRODESTO
               AND EXISTS (SELECT 1
                           FROM PRODUTO P
                           WHERE P.CDPRODORIG = @V_ANTPRODESTO)
         OPEN C_ITEM;
         FETCH NEXT FROM C_ITEM INTO @V_ITEM_CDFILSOLI,  @V_ITEM_CDFILLAN,   @V_ITEM_NRSOLICMP,
                                     @V_ITEM_CDPRODUTO,  @V_ITEM_DTPROGENTR, @V_ITEM_QTSOLIPROD,
                                     @V_ITEM_VRFATOCONV, @V_ITEM_VRARREDCMP;
         WHILE (@@FETCH_STATUS = 0) BEGIN
            SET @V_QTSOLI = @V_ITEM_QTSOLIPROD/@V_ITEM_VRFATOCONV;
            IF @V_VRARREDCMP > 0 BEGIN
               IF (((CAST(@V_QTSOLI AS NUMERIC) / @V_ITEM_VRARREDCMP) - (ROUND(@V_QTSOLI / @V_ITEM_VRARREDCMP, 0, 1))) <> 0)
               BEGIN
                  SET @V_QTSOLI = CEILING(CAST(@V_QTSOLI AS NUMERIC) / @V_ITEM_VRARREDCMP) * @V_ITEM_VRARREDCMP;
               END;
            END;

            UPDATE ITEMSOLI
               SET QTSOLIPROD = @V_QTSOLI
            WHERE CDFILSOLI   = @V_ITEM_CDFILSOLI
              AND CDFILLAN    = @V_ITEM_CDFILLAN
              AND NRSOLICMP   = @V_ITEM_NRSOLICMP
              AND CDPRODUTO   = @V_ITEM_CDPRODUTO
              AND DTPROGENTR  = @V_ITEM_DTPROGENTR;
            FETCH NEXT FROM C_ITEM INTO @V_ITEM_CDFILSOLI,  @V_ITEM_CDFILLAN,   @V_ITEM_NRSOLICMP,
                                        @V_ITEM_CDPRODUTO,  @V_ITEM_DTPROGENTR, @V_ITEM_QTSOLIPROD,
                                        @V_ITEM_VRFATOCONV, @V_ITEM_VRARREDCMP;
         END;
         CLOSE C_ITEM;
         DEALLOCATE C_ITEM;
      END;
      SET @V_ANTPRODESTO = @V_ITEMS_CDPRODORIG;
    END;

    IF @V_QTATUEST >= @V_ITEMS_QTPRODESTO BEGIN
      SET @V_QTATUEST = @V_QTATUEST - @V_ITEMS_QTPRODESTO;

      UPDATE ITEMSOLI
      SET IDITMCONS    = 'S'
      WHERE
         CDFILSOLI  = @V_ITEMS_CDFILSOLI AND
         CDFILLAN   = @V_ITEMS_CDFILLAN AND
         NRSOLICMP  = @V_ITEMS_NRSOLICMP AND
         CDPRODUTO  = @V_ITEMS_CDPRODUTO AND
         DTPROGENTR = @V_ITEMS_DTPROGENTR;
    END
    ELSE BEGIN
      IF @V_IDPRIMVEZ = 1 BEGIN
        SET @V_NOVO_CODIGO = 'SOLICOMP' + @P_CDFILCONS + @P_CDFILCENT;
        EXEC NOVO_CODIGO @V_NOVO_CODIGO, @P_SEQUENCIAL = @V_NRPROXSOLI OUTPUT, @P_QTDE = 1;

        /* INCLUI NOVA SOLICITACAO COM TIPO DE LANCAMENTO = CONSOLIDADA */
        INSERT INTO SOLICITA
        (CDFILSOLI, CDFILLANCA, NRSOLICMP, DTSOLCMP, IDTIPLAN,
         CDFILCENT)
        VALUES
        (@P_CDFILCONS, @P_CDFILCENT, SUBSTRING(@V_NRPROXSOLI, 11, 10),
         CONVERT(DATETIME, CONVERT(VARCHAR, GETDATE(), 103)),
         'C', @P_CDFILCENT);

        SET @V_IDPRIMVEZ = 0;
      END;


      IF NOT ((@V_ITEMS_CDEXISTEPROD IS NOT NULL) AND (@P_IDPRODORIG = 'S')) BEGIN
         SET @V_CONTA = 0;
         SELECT @V_CONTA = COUNT(*)
         FROM ITEMSOLI
         WHERE
          CDFILSOLI  = @P_CDFILCONS AND
          CDFILLAN   = @P_CDFILCENT AND
          NRSOLICMP  = SUBSTRING(@V_NRPROXSOLI, 11, 10) AND
          CDPRODUTO  = @V_ITEMS_CDESTPROD AND
          DTPROGENTR = @V_ITEMS_DTPROGENTR;

         IF @V_CONTA = 0 BEGIN

            SELECT @V_CDFILIOPELOG = MAX(CDFILIOPELOG)
              FROM PARAMPRODSUP
             WHERE CDFILIAL  = @P_CDFILCONS
               AND CDPRODUTO = @V_ITEMS_CDESTPROD
               AND CDFILIOPELOG IS NOT NULL;

            INSERT INTO ITEMSOLI
            (CDFILSOLI,CDFILLAN,NRSOLICMP,CDPRODUTO,DTPROGENTR,
             QTSOLIPROD,DTUTILIZA, CDFILIOPELOG)
            VALUES
            (@P_CDFILCONS, @P_CDFILCENT, SUBSTRING(@V_NRPROXSOLI, 11, 10), @V_ITEMS_CDESTPROD,
             @V_ITEMS_DTPROGENTR,(@V_ITEMS_QTPRODESTO - @V_QTATUEST) / @V_ITEMS_VRFATOCONV,
             @V_ITEMS_DTPROGENTR, @V_CDFILIOPELOG);
             SET @V_QTATUEST = 0;
         END
         ELSE BEGIN
           UPDATE ITEMSOLI
           SET QTSOLIPROD = QTSOLIPROD + ((@V_ITEMS_QTPRODESTO - @V_QTATUEST) / @V_ITEMS_VRFATOCONV)
           WHERE
             CDFILSOLI  = @P_CDFILCONS AND
             CDFILLAN   = @P_CDFILCENT AND
             NRSOLICMP  = SUBSTRING(@V_NRPROXSOLI, 11, 10) AND
             CDPRODUTO  = @V_ITEMS_CDESTPROD AND
             DTPROGENTR = @V_ITEMS_DTPROGENTR;
         END;
      END
      ELSE BEGIN
         SET @V_CONTA = 0;
         SELECT @V_CONTA = COUNT(*)
         FROM ITEMSOLI
         WHERE
          CDFILSOLI  = @P_CDFILCONS AND
          CDFILLAN   = @P_CDFILCENT AND
          NRSOLICMP  = SUBSTRING(@V_NRPROXSOLI, 11, 10) AND
          CDPRODUTO  = @V_ITEMS_CDESTPROD AND
          DTPROGENTR = @V_ITEMS_DTPROGENTR;

         IF @V_CONTA = 0 BEGIN

            SELECT @V_CDFILIOPELOG = MAX(CDFILIOPELOG)
              FROM PARAMPRODSUP
             WHERE CDFILIAL  = @P_CDFILCONS
               AND CDPRODUTO = @V_ITEMS_CDESTPROD
               AND CDFILIOPELOG IS NOT NULL;

            INSERT INTO ITEMSOLI
            (CDFILSOLI,CDFILLAN,NRSOLICMP,CDPRODUTO,DTPROGENTR,
             QTSOLIPROD,DTUTILIZA, CDFILIOPELOG)
            VALUES
            (@P_CDFILCONS, @P_CDFILCENT, SUBSTRING(@V_NRPROXSOLI, 11, 10), @V_ITEMS_CDESTPROD,
             @V_ITEMS_DTPROGENTR,(@V_ITEMS_QTPRODESTO - @V_QTATUEST),
             @V_ITEMS_DTPROGENTR, @V_CDFILIOPELOG);
            SET @V_QTATUEST = 0;
         END
         ELSE BEGIN
           UPDATE ITEMSOLI
           SET QTSOLIPROD = QTSOLIPROD + ((@V_ITEMS_QTPRODESTO - @V_QTATUEST))
           WHERE
             CDFILSOLI  = @P_CDFILCONS AND
             CDFILLAN   = @P_CDFILCENT AND
             NRSOLICMP  = SUBSTRING(@V_NRPROXSOLI, 11, 10) AND
             CDPRODUTO  = @V_ITEMS_CDESTPROD AND
             DTPROGENTR = @V_ITEMS_DTPROGENTR;
         END;
      END;

      UPDATE ITEMSOLI
      SET IDITMCONS = 'S',
          CDFILLANCONS = @P_CDFILCENT,
          CDFILSOLCONS = @P_CDFILCONS,
          NRSOLCMPCONS = SUBSTRING(@V_NRPROXSOLI, 11, 10),
          CDPRODUTCONS = @V_ITEMS_CDESTPROD,
          DTPROGENCONS = @V_ITEMS_DTPROGENTR
      WHERE
         CDFILSOLI  = @V_ITEMS_CDFILSOLI AND
         CDFILLAN   = @V_ITEMS_CDFILLAN AND
         NRSOLICMP  = @V_ITEMS_NRSOLICMP AND
         CDPRODUTO  = @V_ITEMS_CDPRODUTO AND
         DTPROGENTR = @V_ITEMS_DTPROGENTR;

      SET @P_NRSOLICMP = SUBSTRING(@V_NRPROXSOLI, 11, 10);
    END;
    FETCH NEXT FROM C_ITEMS INTO @V_ITEMS_CDFILSOLI,  @V_ITEMS_CDFILLAN,   @V_ITEMS_NRSOLICMP,
                                 @V_ITEMS_CDPRODUTO,  @V_ITEMS_DTPROGENTR, @V_ITEMS_VRFATOCONV,
                                 @V_ITEMS_QTPRODESTO, @V_ITEMS_CDPRODESTO, @V_ITEMS_CDPRODORIG,
                                 @V_ITEMS_CDEXISTEPROD, @V_ITEMS_CDESTPROD;
  END;
  CLOSE C_ITEMS;
  DEALLOCATE C_ITEMS;

    /* Calcula calendrio de compras para as periodicidades que ainda no possuem */

  DECLARE C_PERIODNCALC CURSOR FOR
      SELECT DISTINCT
         PERFILPROD.CDPERIENTR,
         CONVERT(DATETIME, CONVERT(VARCHAR, ITEMSOLI.DTPROGENTR-DAY(ITEMSOLI.DTPROGENTR)+1, 103), 103) AS DTPREVCONS
      FROM
        (SELECT
            ITEMSOLI.DTPROGENTR,
            P.CDARVPROD
         FROM
            ITEMSOLI,
            PRODUTO P
         WHERE
            ITEMSOLI.CDFILSOLI = @P_CDFILCONS AND
            ITEMSOLI.CDFILLAN = @P_CDFILCENT AND
            ITEMSOLI.NRSOLICMP = SUBSTRING(@V_NRPROXSOLI, 11, 10) AND
            ITEMSOLI.CDPRODUTO = P.CDPRODUTO) ITEMSOLI,
         PERFILPROD,
         PRODUTO PROD_PER
      WHERE
        (PERFILPROD.CDPRODUTO = PROD_PER.CDPRODUTO) AND
        (PERFILPROD.CDFILIAL  = @P_CDFILCONS) AND
        (PROD_PER.CDARVPROD = (SELECT
                                  MAX(PRODUTO.CDARVPROD)
                               FROM
                                  PERFILPROD A,
                                  PRODUTO
                               WHERE
                                  (A.CDPRODUTO = PRODUTO.CDPRODUTO) AND
                                  (A.CDFILIAL = @P_CDFILCONS) AND
                                  (CHARINDEX(PRODUTO.CDARVPROD, ITEMSOLI.CDARVPROD) = 1))) AND
        (PERFILPROD.CDPERIENTR NOT IN (SELECT
                                          CDPERIENTR
                                       FROM
                                          CALECOMP
                                       WHERE
                                          (CALECOMP.CDFILIAL = @P_CDFILCONS) AND
                                          (SUBSTRING(CONVERT(VARCHAR, CALECOMP.DTENTREGA,103), 4, 7) =
                                           SUBSTRING(CONVERT(VARCHAR, ITEMSOLI.DTPROGENTR,103), 4, 7)) ));
  OPEN C_PERIODNCALC;
  FETCH NEXT FROM C_PERIODNCALC INTO @V_PERIODNCALC_CDPERIENTR,
                                     @V_PERIODNCALC_DTPREVCONS;
  WHILE (@@FETCH_STATUS = 0) BEGIN
     EXEC CALC_DATA_ENTR_MES @P_CDFILCONS,
                             @V_PERIODNCALC_CDPERIENTR,
                             @V_PERIODNCALC_DTPREVCONS,
                             1,
                             @V_DTPROCESSA;

     FETCH NEXT FROM C_PERIODNCALC INTO @V_PERIODNCALC_CDPERIENTR,
                                        @V_PERIODNCALC_DTPREVCONS;
  END;
  CLOSE C_PERIODNCALC;
  DEALLOCATE C_PERIODNCALC;

  DECLARE C_ITEMANTECIP CURSOR FOR
      SELECT
         ITEMSOLI.CDPRODUTO,
         PRODUTO.CDARVPROD,
         ITEMSOLI.DTPROGENTR,
         ITEMSOLI.QTSOLIPROD,
         PERFILPROD.CDPERIENTR
      FROM
         ITEMSOLI, PRODUTO, PRODUTO PRODPERIENTR, PERFILPROD
      WHERE ITEMSOLI.CDFILSOLI     = @P_CDFILCONS
        AND ITEMSOLI.CDFILLAN      = @P_CDFILCENT
        AND ITEMSOLI.NRSOLICMP     = SUBSTRING(@V_NRPROXSOLI, 11, 10)
        AND ITEMSOLI.CDPRODUTO     = PRODUTO.CDPRODUTO
        AND PERFILPROD.CDPRODUTO   = PRODPERIENTR.CDPRODUTO
        AND PERFILPROD.CDFILIAL    = @P_CDFILCONS
        AND PRODPERIENTR.CDARVPROD = (SELECT
                                         MAX(B.CDARVPROD)
                                      FROM
                                         PERFILPROD A,
                                         PRODUTO B
                                      WHERE A.CDPRODUTO = B.CDPRODUTO
                                       AND  A.CDFILIAL  = @P_CDFILCONS
                                       AND  CHARINDEX(B.CDARVPROD, PRODUTO.CDARVPROD) = 1)
      ORDER BY PRODUTO.CDARVPROD, ITEMSOLI.DTPROGENTR;
  OPEN C_ITEMANTECIP;
  FETCH NEXT FROM C_ITEMANTECIP INTO @V_ITEMANTECIP_CDPRODUTO,
                                     @V_ITEMANTECIP_CDARVPROD,
                                     @V_ITEMANTECIP_DTPROGENTR,
                                     @V_ITEMANTECIP_QTSOLIPROD,
                                     @V_ITEMANTECIP_CDPERIENTR;

  WHILE (@@FETCH_STATUS = 0) BEGIN
     SET @V_DTENTREGA = NULL;

     IF @P_CONSLIMSEG = 'S' BEGIN
        SELECT @V_NRDIASEG = MAX(NRLIMISEGU)
          FROM LIMISEGPROD, PRODUTO, PRODUTO PRODNIVELCOMPL
         WHERE CDFILIAL = @P_CDFILCONS
           AND LIMISEGPROD.CDPRODUTO = PRODUTO.CDPRODUTO
           AND PRODNIVELCOMPL.CDPRODUTO = @V_ITEMANTECIP_CDPRODUTO
           AND PRODNIVELCOMPL.CDARVPROD LIKE PRODUTO.CDARVPROD+'%'
           AND PRODUTO.CDARVPROD = (SELECT MAX(B.CDARVPROD)
                                          FROM LIMISEGPROD A, PRODUTO B
                                         WHERE A.CDPRODUTO = B.CDPRODUTO
                                           AND A.CDFILIAL  = @P_CDFILCONS
                                           AND CHARINDEX(B.CDARVPROD, PRODNIVELCOMPL.CDARVPROD) = 1);
     END
     ELSE BEGIN
        SET @V_NRDIASEG = 0;
     END;
     SET @V_NOVADATA = @V_ITEMANTECIP_DTPROGENTR - @V_NRDIASEG;
     SELECT
        @V_DTENTREGA = MAX(CALECOMP.DTENTREGA)
     FROM
        CALECOMP
     WHERE CALECOMP.CDFILIAL    = @P_CDFILCONS
       AND CALECOMP.CDPERIENTR  = @V_ITEMANTECIP_CDPERIENTR
       AND CALECOMP.DTENTREGA  <= @V_NOVADATA;

     IF (@V_DTENTREGA IS NOT NULL) AND (@V_DTENTREGA <> @V_ITEMANTECIP_DTPROGENTR) BEGIN
        UPDATE ITEMSOLI
        SET QTSOLIPROD = QTSOLIPROD + @V_ITEMANTECIP_QTSOLIPROD
        WHERE CDFILSOLI = @P_CDFILCONS
          AND CDFILLAN = @P_CDFILCENT
          AND NRSOLICMP = SUBSTRING(@V_NRPROXSOLI, 11, 10)
          AND CDPRODUTO = @V_ITEMANTECIP_CDPRODUTO
          AND DTPROGENTR = @V_DTENTREGA;

        IF @@ROWCOUNT = 0 BEGIN

           SELECT @V_CDFILIOPELOG = MAX(CDFILIOPELOG)
             FROM PARAMPRODSUP
            WHERE CDFILIAL  = @P_CDFILCONS
              AND CDPRODUTO = @V_ITEMANTECIP_CDPRODUTO
              AND CDFILIOPELOG IS NOT NULL;

           INSERT INTO ITEMSOLI
            (CDFILSOLI,
             CDFILLAN,
             NRSOLICMP,
             CDPRODUTO,
             DTPROGENTR,
             QTSOLIPROD,
             DTUTILIZA,
             IDITMCONS,
             TXOBSITMSOL,
             IDBAIXITMSOL,
             QTTRANSFSOL,
             NRLOTEESTQ,
             NRSUBLOTE,
             CDPRODPROD,
             CDFILLANCONS,
             CDFILSOLCONS,
             NRSOLCMPCONS,
             CDPRODUTCONS,
             DTPROGENCONS,
             CDFILIOPELOG)
           SELECT
             CDFILSOLI,
             CDFILLAN,
             NRSOLICMP,
             CDPRODUTO,
             @V_DTENTREGA,
             QTSOLIPROD,
             DTUTILIZA,
             IDITMCONS,
             TXOBSITMSOL,
             IDBAIXITMSOL,
             QTTRANSFSOL,
             NRLOTEESTQ,
             NRSUBLOTE,
             CDPRODPROD,
             CDFILLANCONS,
             CDFILSOLCONS,
             NRSOLCMPCONS,
             CDPRODUTCONS,
             DTPROGENCONS,
             @V_CDFILIOPELOG
           FROM
             ITEMSOLI
           WHERE
             CDFILSOLI = @P_CDFILCONS AND
             CDFILLAN = @P_CDFILCENT AND
             NRSOLICMP = SUBSTRING(@V_NRPROXSOLI, 11, 10) AND
             CDPRODUTO = @V_ITEMANTECIP_CDPRODUTO AND
             DTPROGENTR = @V_ITEMANTECIP_DTPROGENTR;
        END;

        UPDATE ITEMSOLI
        SET DTPROGENCONS = @V_DTENTREGA
        WHERE CDFILLANCONS = @P_CDFILCONS
          AND CDFILSOLCONS = @P_CDFILCENT
          AND NRSOLCMPCONS = SUBSTRING(@V_NRPROXSOLI, 11, 10)
          AND CDPRODUTCONS = @V_ITEMANTECIP_CDPRODUTO
          AND DTPROGENCONS = @V_ITEMANTECIP_DTPROGENTR;

        DELETE FROM ITEMSOLI
        WHERE
           CDFILSOLI = @P_CDFILCONS AND
           CDFILLAN = @P_CDFILCENT AND
           NRSOLICMP = SUBSTRING(@V_NRPROXSOLI, 11, 10) AND
           CDPRODUTO = @V_ITEMANTECIP_CDPRODUTO AND
           DTPROGENTR = @V_ITEMANTECIP_DTPROGENTR;
     END;
     FETCH NEXT FROM C_ITEMANTECIP INTO @V_ITEMANTECIP_CDPRODUTO,
                                        @V_ITEMANTECIP_CDARVPROD,
                                        @V_ITEMANTECIP_DTPROGENTR,
                                        @V_ITEMANTECIP_QTSOLIPROD,
                                        @V_ITEMANTECIP_CDPERIENTR;
  END;
  CLOSE C_ITEMANTECIP;
  DEALLOCATE C_ITEMANTECIP;

  /* Considera fator de converso do produto origem */
  IF (@P_IDPRODORIG = 'S') AND (@V_ANTPRODESTO <> ' ') BEGIN
     DECLARE C_ITEM CURSOR FOR
         SELECT
            ITEMSOLI.CDFILSOLI,
            ITEMSOLI.CDFILLAN,
            ITEMSOLI.NRSOLICMP,
            ITEMSOLI.CDPRODUTO,
            ITEMSOLI.DTPROGENTR,
            ITEMSOLI.QTSOLIPROD,
            ISNULL(PRODUTO.VRFATOCONV, 1),
            ISNULL(VRARREDCMP, 0) VRARREDCMP
         FROM
            ITEMSOLI, PRODUTO
         WHERE ITEMSOLI.CDPRODUTO = PRODUTO.CDPRODUTO
           AND ITEMSOLI.CDFILSOLI = @P_CDFILCONS
           AND ITEMSOLI.CDFILLAN  = @P_CDFILCENT
           AND ITEMSOLI.NRSOLICMP = SUBSTRING(@V_NRPROXSOLI, 11, 10)
           AND ITEMSOLI.CDPRODUTO = @V_ANTPRODESTO
           AND EXISTS (SELECT 1
                       FROM PRODUTO P
                       WHERE P.CDPRODORIG = @V_ANTPRODESTO)
     OPEN C_ITEM;
     FETCH NEXT FROM C_ITEM INTO @V_ITEM_CDFILSOLI,  @V_ITEM_CDFILLAN,   @V_ITEM_NRSOLICMP,
                                 @V_ITEM_CDPRODUTO,  @V_ITEM_DTPROGENTR, @V_ITEM_QTSOLIPROD,
                                 @V_ITEM_VRFATOCONV, @V_ITEM_VRARREDCMP;
     WHILE (@@FETCH_STATUS = 0) BEGIN
        SET @V_QTSOLI = @V_ITEM_QTSOLIPROD/@V_ITEM_VRFATOCONV;
        IF @V_VRARREDCMP > 0 BEGIN
           IF (((CAST(@V_QTSOLI AS NUMERIC) / @V_ITEM_VRARREDCMP) - (ROUND(@V_QTSOLI / @V_ITEM_VRARREDCMP, 0, 1))) <> 0)
           BEGIN
              SET @V_QTSOLI = CEILING(CAST(@V_QTSOLI AS NUMERIC) / @V_ITEM_VRARREDCMP) * @V_ITEM_VRARREDCMP;
           END;
        END;

        UPDATE ITEMSOLI
           SET QTSOLIPROD = @V_QTSOLI
        WHERE CDFILSOLI   = @V_ITEM_CDFILSOLI
          AND CDFILLAN    = @V_ITEM_CDFILLAN
          AND NRSOLICMP   = @V_ITEM_NRSOLICMP
          AND CDPRODUTO   = @V_ITEM_CDPRODUTO
          AND DTPROGENTR  = @V_ITEM_DTPROGENTR;
        FETCH NEXT FROM C_ITEM INTO @V_ITEM_CDFILSOLI,  @V_ITEM_CDFILLAN,   @V_ITEM_NRSOLICMP,
                                    @V_ITEM_CDPRODUTO,  @V_ITEM_DTPROGENTR, @V_ITEM_QTSOLIPROD,
                                    @V_ITEM_VRFATOCONV, @V_ITEM_VRARREDCMP;
     END;
     CLOSE C_ITEM;
     DEALLOCATE C_ITEM;
  END;
END;
GO


-- VERSAO --
EXEC ATUVEROBJ 'CONSOLIDA_SOLICITA','5.05.061';
GO

