/**
 * Service to create widgets to handle layout
 *
 * @class
 * @since 3.0.0
 *
 * @param {RestEngine}      restEngine      Service to make http requests
 * @param {TemplateManager} templateManager Service to handle templates
 * @param {MetaDataFactory} metaDataFactory Service to factory models from metadata
 */
function ZhLayoutWidgetsBuilder(restEngine, templateManager, metaDataFactory){

    function getWidgetJSON(widgetName){
        var widgetPath = 'zh-layout-preferences#json/widgets/'+widgetName+'.json';
        return restEngine.requestMetaData(widgetPath).then(function(response){
            return response.data;
        });
    }

    /**
     * Retrieve a factored widget
     *
     * @since 3.0.0
     *
     * @param  {String} widgetName Widget name
     *
     * @return {Widget}            Factored widget
     */
    this.factoryWidget = function(widgetName){
        return getWidgetJSON(widgetName).then(function(widget){
            return metaDataFactory.widgetFactory(widget, templateManager.container);
        });
    };

    /**
     * Retrieve factored layout configuration widget
     *
     * @since 3.0.0
     *
     * @return {Widget} Layout configuration widget
     */
    this.factoryLayoutConfigurationWidget = function(){
        return this.factoryWidget('config');
    };

    /**
     * Retrieve factored create layout widget
     *
     * @since 3.0.0
     *
     * @return {Widget} Create layout widget
     */
    this.factoryCreateLayoutWidget = function(){
        return this.factoryWidget('createLayout');
    };

    /**
     * Retrieve factored import layout widget
     *
     * @since 3.0.0
     *
     * @return {Widget} Import layout widget
     */
    this.factoryImportLayoutWidget = function(){
        return this.factoryWidget('importLayout');
    };

    /**
     * Retrieve factored build layout options widget
     *
     * @since 3.0.0
     *
     * @return {Widget} Build layout options widget
     */
    this.factoryBuildLayoutOptionsWidget = function(){
        return this.factoryWidget('createLayoutOptions');
    };

    /**
     * Retrieve factored build layout field options widget
     *
     * @since 3.0.0
     *
     * @return {Widget} Build layout field options widget
     */
    this.factoryBuildLayoutFieldOptionsWidget = function(){
        return this.factoryWidget('createLayoutFieldOptions');
    };

}

Configuration(function(ContextRegister){
    ContextRegister.register('ZhLayoutWidgetsBuilder', ZhLayoutWidgetsBuilder);
});